/*
 * Copyright 2009 Jonathan Rubin
 * This file is part of Camus.
 *
 * Camus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Camus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Camus.  If not, see <http://www.gnu.org/licenses/>.
 */

package cbr.framework;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * <p>A case consists of indexed and non-indexed features, a solution and the last similarity value that was computed for this case.</p>
 *
 */
public class Case implements Comparable<Case> {
	private Map<String, Feature> features;
	private Solution solution;
	private Outcome outcome;
	private double similarity;
	
	/**
	 * Create an empty case.
	 */
	public Case(){
		features = new LinkedHashMap<String, Feature>();
		outcome = new Outcome();
	}
	
	/**
	 * Create a case with features contained in a <code>Map</code>.
	 * @param allFeatures a <code>Map</code> which holds <code>Features</code> with a name key.
	 */
	public Case(final Map<String, Feature> allFeatures){
		features = new LinkedHashMap<String, Feature>();
		features.putAll(allFeatures);
		outcome = new Outcome();
	}
	
	/**
	 * Add a <code>feature</code> to the <code>case</code>.
	 * @param f the additional <code>Feature</code>.
	 */
	public void addFeature(final Feature f){
		features.put(f.getName(), f);
	}
	
	/**
	 * Set the <code>Solution</code> for this <code>case</code>.
	 * @param solution the solution
	 */
	public void setSolution(final Solution solution){
		this.solution = solution;
	}
	
	/**
	 * Retrieve the <code>solution</code> for this <code>case</code>.
	 * @return the solution
	 */
	public Solution getSolution(){
		return solution;
	}
	
	/**
	 * Set the <code>Outcome</code> value for this <code>case</code>.
	 * @param value the outcome value
	 */
	public void setOutcomeValue(final double value){
		outcome.setValue(value);
	}
	
	/**
	 * Retrieve the <code>outcome</code> value for this <code>case</code>.
	 * @return the outcome as a double
	 */
	public double getOutcomeValue(){
		return outcome.getValue();
	}
	
	/**
	 * Iterate over the features, solution and current similarity and display each of their values.
	 * @return a description <code>String</code>
	 */
	public String toString(){
		String desc = "";
		for(final Feature f : features.values()){
			desc += f.toString() + " ";
		}
		desc += "\tSolution: " + solution.toString();
		desc += "\tSimilarity: " + similarity;
		desc += "\t\tOutcome: " + outcome.getValue();
		return desc;
	}
	
	/**
	 * Formats a <code>String</code> to describe this <code>case</code>, so it can be represented in flat file format.
	 * Used to retain new target cases in the case base. 
	 * @return the formatted string
	 * @throws IllegalArgumentException if the <code>case</code> does not contain a <code>solution</code>
	 */
	protected String getFlatFileFormat(){
		if(solution == null){
			throw new IllegalArgumentException("Cannot get Flat File Format as Case does not have a Solution.");
		}
		
		String desc = "";
		for(final Feature f : features.values()){
			desc += f.getName() + ":" + f.getValueAsString() + ":" + (f.isIndexed() ? "true" : "false") + "/";
		}
		desc += solution.toString();
		desc += "|" + outcome.getValue();
		return desc;
	}
	
	/**
	 * Get the last similarity value that was derived.
	 * @return the latest similarity value
	 */
	public double getSimilarity() {
		return similarity;
	}
	
	/**
	 * Calculate the similarity between a target case and this case by cycling
	 * through each feature using a pre-specified similarity metric.
	 * @param targetCase a <code>case</code> which must use <code>TargetFeatures</code> to specify a similarity metric
	 */
	protected void calculateSimilarity(final Case targetCase){
		double sim = 0.0;
		double total = 0.0;
		
		for(final Feature f : targetCase.features.values()){
			if(!(f instanceof TargetFeature)){
				throw new IllegalArgumentException("A TargetCase must use TargetFeatures: " + f.getName());
			}
			final TargetFeature targetFeature = (TargetFeature)f;
			final Feature sourceFeature = features.get(targetFeature.getName());
			
			// If the feature is an indexed feature, then use the similarity metric specified to compare the
			// target to the source and add this to the similarity measurement
			if(sourceFeature.isIndexed()){
				sim += targetFeature.getSimilarityMetric().compare(targetFeature, sourceFeature) * targetFeature.getWeight();
				total += targetFeature.getWeight();
			}
		}
		similarity = sim / total;
	}
	
	/**
	 * Converts a target case to a source case by retrieving the <code>TargetFeatures</code> of the case and converting them
	 * to the super type <code>Feature</code>. This method should be called before retaining a target case in the case base. 
	 */
	protected void convertTargetCaseToSourceCase(){
		final Collection<Feature> targetFeatures = features.values();
		features = new LinkedHashMap<String, Feature>();
		
		for(final Feature f : targetFeatures){
			features.put(f.getName(), new Feature(f.getName(), new Value(f.getValueAsString()), f.isIndexed()));
		}
	}
	
	/**
	 * Override the compareTo() method so that <code>Cases</code> can be sorted according to descending similarity.
	 * @param other the <code>Case</code> to compare with.
	 * @return an <code>int</code> which represents the <code>cases</code> order in the sort.
	 */
	@Override
	public int compareTo(Case other){
		// Makes use of the Double Wrapper class to do the comparisons.
		return -(new Double(similarity).compareTo(other.similarity));
	}
}
