/*
 * Copyright 2009 Jonathan Rubin
 * This file is part of Camus.
 *
 * Camus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Camus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Camus.  If not, see <http://www.gnu.org/licenses/>.
 */

package cbr.similarity;

import cbr.framework.Feature;
import cbr.framework.SimilarityMetric;

/**
 * <p>Provides implementation for a Euclidean Distance Metric. The following formula is used:
 * <br/><code>Similarity = 1 - (|value1 - value2| / MAX_VALUE)</code><br/>
 * Which produces a normalised, inverted similarity value between two values.</p> 
 */
public class EuclideanDistanceMetric implements SimilarityMetric {
	private double max;
	
	/**
	 * Creates a Euclidean Distance Metric given a value for the maximum distance between two values.
	 * @param max the maximum value between two feature values
	 */
	public EuclideanDistanceMetric(final double max){
		this.max = max;
	}
	
	/**
	 * Returns the maximum value, used in the equation for this metric. 
	 * @return the maximum value
	 */
	public double getMax() {
		return max;
	}
	
	/**
	 * Specify the maximum difference between two numerical values
	 * @param max the maximum value
	 */
	public void setMax(double max) {
		this.max = max;
	}

	/**
	 * Derives the normalised, inverted 1-dimensional Euclidean distance between two numerical feature values
	 * @return 1.0 is exactly similar, 0.0 is entirely dissimilar
	 */
	@Override
	public double compare(Feature f1, Feature f2){
		final double value1 = Double.parseDouble(f1.getValueAsString());
		final double value2 = Double.parseDouble(f2.getValueAsString());
		
		return 1 - (Math.abs(value1 - value2) / max);
	}
}
